import { createSlice } from '@reduxjs/toolkit'
import { createAppAsyncThunk } from '../createAppAsyncThunk'
import IResponse from '../../interfaces/IResponse'
import IComment from '../../interfaces/IComment'
import ICommentState from './ICommentState'
import { commentsApi } from '../../api/commentApi'
import ICommentDto from '../../interfaces/ICommentDto'


const namespace = 'comments'

export const getCommentsByPost = createAppAsyncThunk(
    `${namespace}/getCommentsByPost`,
    async (postId: string): Promise<IResponse<IComment[] | undefined>> => {
        return commentsApi.getCommentsByPost(postId)
    }
)

export const createComment = createAppAsyncThunk(
    `${namespace}/createComment`,
    async (comment: ICommentDto) => {
        return commentsApi.createComment(comment)
    }
)

export const deleteCommentById = createAppAsyncThunk(
    `${namespace}/deleteCommentById`,
    async (id: string) => {
        return commentsApi.deleteCommentById(id)
    }
)

export const commentsSlice = createSlice({
    name: namespace,
    initialState: {
        comments: [] as IComment[],
        loadingComments: false,
        messageComments: ''
    } as ICommentState,
    reducers: {},
    extraReducers: (builder) => {
        builder
            .addCase(getCommentsByPost.rejected, (state) => {
                state.loadingComments = false
            })
            .addCase(getCommentsByPost.pending, (state) => {
                state.loadingComments = true
            })
            .addCase(getCommentsByPost.fulfilled, (state, action) => {
                state.loadingComments = false
                state.comments = action.payload.result as IComment[]
                state.messageComments = action.payload.message
            })
            .addCase(createComment.rejected, (state) => {
                state.loadingComments = false
            })
            .addCase(createComment.pending, (state) => {
                state.loadingComments = true
            })
            .addCase(createComment.fulfilled, (state, action) => {
                state.loadingComments = false
                state.messageComments = action.payload.message
            })
            .addCase(deleteCommentById.rejected, (state) => {
                state.loadingComments = false
            })
            .addCase(deleteCommentById.pending, (state) => {
                state.loadingComments = true
            })
            .addCase(deleteCommentById.fulfilled, (state, action) => {
                state.loadingComments = false
                state.messageComments = action.payload.message
            })
    }
})