import { createSlice } from '@reduxjs/toolkit'
import IPost from '../../interfaces/IPost'
import IPostState from './IPostState'
import { createAppAsyncThunk } from '../createAppAsyncThunk'
import { postApi } from '../../api/postApi'
import IResponse from '../../interfaces/IResponse'


const namespace = 'posts'

export const getPosts = createAppAsyncThunk(
    `${namespace}/getPosts`,
    async (): Promise<IResponse<IPost[] | undefined>> => {
        return postApi.getPosts()
    }
)

export const getPostById = createAppAsyncThunk(
    `${namespace}/getPostById`,
    async (id: string): Promise<IResponse<IPost | undefined>> => {
        return postApi.getPostById(id)
    }
)

export const createPost = createAppAsyncThunk(
    `${namespace}/createPost`,
    async (post: FormData) => {
        return postApi.createPost(post)
    }
)

export const deletePostById = createAppAsyncThunk(
    `${namespace}/deletePostById`,
    async (id: string) => {
        return postApi.deletePostById(id)
    }
)

export const postsSlice = createSlice({
    name: namespace,
    initialState: {
        posts: [] as IPost[],
        post: {},
        loadingPosts: false,
        messagePosts: ''
    } as IPostState,
    reducers: {},
    extraReducers: (builder) => {
        builder
            .addCase(getPosts.rejected, (state) => {
                state.loadingPosts = false
            })
            .addCase(getPosts.pending, (state) => {
                state.loadingPosts = true
            })
            .addCase(getPosts.fulfilled, (state, action) => {
                state.loadingPosts = false
                state.posts = action.payload.result as IPost[]
                state.messagePosts = action.payload.message
            })
            .addCase(getPostById.rejected, (state) => {
                state.loadingPosts = false
            })
            .addCase(getPostById.pending, (state) => {
                state.loadingPosts = true
            })
            .addCase(getPostById.fulfilled, (state, action) => {
                state.loadingPosts = false
                state.post = action.payload.result as IPost
                state.messagePosts = action.payload.message
            })
            .addCase(createPost.rejected, (state) => {
                state.loadingPosts = false
            })
            .addCase(createPost.pending, (state) => {
                state.loadingPosts = true
            })
            .addCase(createPost.fulfilled, (state, action) => {
                state.loadingPosts = false
                state.messagePosts = action.payload.message
            })
            .addCase(deletePostById.rejected, (state) => {
                state.loadingPosts = false
            })
            .addCase(deletePostById.pending, (state) => {
                state.loadingPosts = true
            })
            .addCase(deletePostById.fulfilled, (state, action) => {
                state.loadingPosts = false
                state.messagePosts = action.payload.message
            })
    }
})