import { createSlice } from "@reduxjs/toolkit";
import { userApi } from "../../api/userApi";
import IUser from "../../interfaces/IUser";
import IUserCreateDto from "../../interfaces/IUserCreateDto";
import { createAppAsyncThunk } from "../createAppAsyncThunk";
import IUsersState from './IUserState'

const namespace = 'users'


export const login = createAppAsyncThunk(
    `${namespace}/login`,
    async (user: IUserCreateDto) => {
        return userApi.login(user)
    }
)

export const register = createAppAsyncThunk(
    `${namespace}/register`,
    async (user: IUserCreateDto) => {
        return userApi.register(user)
    }
)

export const checkToken = createAppAsyncThunk(
    `${namespace}/checkToken`,
    async () => {
        return userApi.checkToken()
    }
)


export const usersSlice = createSlice({
    name: namespace,
    initialState: {
        user: {} as IUser,
        isAuth: false,
        loadingUser: false,
        messageUser: ''
    } as IUsersState,
    reducers: {},
    extraReducers: (builder) => {
        builder
            .addCase(login.rejected, (state) => {
                state.loadingUser = false
            })
            .addCase(login.pending, (state) => {
                state.loadingUser = true
            })
            .addCase(login.fulfilled, (state, action) => {
                state.loadingUser = false
                const user = action.payload.result
                state.user = user
                state.messageUser = action.payload.message
                if (user) {
                    localStorage.setItem('token', user.token)
                    state.isAuth = true
                }
            })
            .addCase(register.rejected, (state) => {
                state.loadingUser = false
            })
            .addCase(register.pending, (state) => {
                state.loadingUser = true
            })
            .addCase(register.fulfilled, (state, action) => {
                state.loadingUser = false
                const user = action.payload.result
                state.user = user
                state.messageUser = action.payload.message
                if (user) {
                    localStorage.setItem('token', user.token)
                    state.isAuth = true
                }
            })
            .addCase(checkToken.rejected, (state) => {
                state.loadingUser = false
            })
            .addCase(checkToken.pending, (state) => {
                state.loadingUser = true
            })
            .addCase(checkToken.fulfilled, (state, action) => {
                state.loadingUser = false
                const user = action.payload.result
                state.user = user
                state.messageUser = action.payload.message
                if (user) {
                    state.isAuth = true
                } else {
                    state.isAuth = false
                }
            })
    }
})

export const {} = usersSlice.actions